function [decision,conf] = cep_nonlingam (cause,effect,methodpars,outfile)
% function [decision,conf] = cep_nonlingam (cause,effect,methodpars,outfile)
%
% Wrapper that runs NONLINGAM and writes results to outfile (see also cep_template).
%
% It fits a GP to the data and runs an HSIC test on the residuals and the input.
% It decides on the direction having the higher HSIC p-value.
% As confidence value, it returns the maximum of the two HSIC p-values.
%
% INPUT:   methodpars       parameters
%            .nrperm          number of permutations to use with HSIC test,
%                             0 means gamma approximation (default: 0)
%            .gaussianize     whether to gaussianize the cause before fitting the GP (default: 0)
%            .FITC            if nonzero, uses this amount of points for FITC approximation (using linear grid)
%            .splitdata       whether to split the data into a training set (for regression) and a test set (for independence testing)
%            .evaluation      model selection criterion: either 'pHSIC' (default), or 'HSIC'
%
% Copyright (c) 2008-2014  Joris Mooij  <j.m.mooij@uva.nl>
% All rights reserved.  See the file LICENSE for license terms.

  if size(cause,2) ~= 1 || size(effect,2) ~= 1
    decision = nan;
    conf = -Inf;
  else
    % set default parameters
    if ~isfield(methodpars,'nrperm')
      methodpars.nrperm = 0;
    end
    if ~isfield(methodpars,'gaussianize')
      methodpars.gaussianize = 0;
    end
    if ~isfield(methodpars,'FITC')
      methodpars.FITC = 0;
    end
    if ~isfield(methodpars,'splitdata')
      methodpars.splitdata = 0;
    end
    if ~isfield(methodpars,'evaluation')
      methodpars.evaluation = 'pHSIC';
    end

    % set GP pars
    gppars.meanf = 'meanAffine';

    forward = GP_plus_HSIC(cause,effect,methodpars,gppars);
    backward = GP_plus_HSIC(effect,cause,methodpars,gppars);

    % take decision
    decision = 0;
    if forward.conf > backward.conf
      decision = 1;
    elseif forward.conf < backward.conf
      decision = -1;
    end

    % calculate confidence
    conf = max([forward.conf, backward.conf]);

    % save results
    save(outfile,'cause','effect','methodpars','forward','backward','decision','conf','-v7.3');
  end

return


function results = GP_plus_HSIC(input,output,methodpars,gppars)

    % gaussianize input if requested
    if methodpars.gaussianize
      input = gaussianize(input);
    end

    if 0 % partition data for cross-validation
      cvk = 1;
      [train,test] = cvfolds(length(input),cvk);
      results.train = cell(cvk,1);
      results.test = cell(cvk,1);
      results.fit = zeros(length(input),1);
      for k = 1:cvk
        results.train{k} = train_gp(input(train{k}),output(train{k}),gppars);
        results.test{k} = test_gp(input(test{k}),results.train{k}.model,gppars);
        results.fit(test{k}) = results.test{k}.Ytest;
        results.stddev_noise(test{k}) = sqrt(results.test{k}.Ytestvar);
      end
      for k = 1:cvk
        p = fasthsic(input(test{k}),output(test{k}) - results.test{k}.Ytest,0.0,0.0,methodpars.nrperm)
      end
      keyboard
    end

    % split data into train and test set, if requested
    if methodpars.splitdata
      [train,test] = cvfolds(length(input),2);
      train = train{1}; test = test{1};
    else
      train = [1:length(input)];
      test  = [1:length(input)];
    end

    % fit GP
    if methodpars.FITC ~= 0
      gppars.FITC = linspace(min(input),max(input),methodpars.FITC)';
    end
    results.train = train;
    results.test = test;
    results.result = train_gp(input(train),output(train),gppars);
    results.fit = results.result.Yfit;
    results.stddev_noise = sqrt(results.result.Yvar);

    % evaluate HSIC p-value of input versus residuals
    results.predict = test_gp(input(test),results.result.model,gppars);
    [results.pHSIC,results.HSIC] = fasthsic(input(test),output(test) - results.predict.Ytest,0.0,0.0,methodpars.nrperm);

    if strcmp(methodpars.evaluation,'pHSIC')
      results.conf = results.pHSIC;
    elseif strcmp(methodpars.evaluation,'HSIC')
      results.conf = -results.HSIC;
    else
      error('Unknown evaluation method');
    end

return
