function [cause,effect,weight] = load_pair (pair,preprocessing,webdavdir)
% function [cause,effect,weight] = load_pair (pair,preprocessing,webdavdir)
%
% Loads a cause-effect pair and applies preprocessing to the data
%
% INPUT:   pair           number of the cause effect pair (e.g., 1)
%          preprocessing  the preprocessing to apply to the data
%            .normalize     false:  do not normalize data
%                           true:   make mean == 0 and std == 1 (default: true)
%            .maxN          maximum number of data points for subsampling,
%                           0 for all available data points (default: inf)
%            .randseed      random number seed for permuting data points (default: 42)
%            .undisc        whether the data should be 'undiscretized'
%                             (which adds small amounts of noise to the
%                              components of data points whose values
%                              coincide with those of other data points) (default: false)
%            .disturbance   standard deviation of Gaussian noise added to
%                           the normalized, undiscretized variables (default: 0)
%          webdavdir      directory path to nonlinngam/webdav
%
% OUTPUT:  cause          preprocessed cause data (N*dX vector)
%          effect         preprocessed effect data (N*dY vector)
%          weight         weight of the pair in overall accuracy score
%
% Copyright (c) 2008-2014  Joris Mooij  <j.m.mooij@uva.nl>
% All rights reserved.  See the file LICENSE for license terms.

  metadata = load(sprintf('%s/pairmeta.txt',webdavdir));
  if metadata(pair,1) ~= pair 
    error('Assumption on metadata not satisfied');
  else
    % set default parameters
    if nargin < 2
      preprocessing = struct;
    end
    if ~isfield(preprocessing,'normalize')
      preprocessing.normalize = true;
    end
    if ~isfield(preprocessing,'maxN')
      preprocessing.maxN = inf;
    end
    if ~isfield(preprocessing,'randseed')
      preprocessing.randseed = 42;
    end
    if ~isfield(preprocessing,'undisc')
      preprocessing.undisc = false;
    end
    if ~isfield(preprocessing,'disturbance')
      preprocessing.disturbance = false;
    end

    % load data
    filename = sprintf('%s/pair%04d.txt',webdavdir,pair);
    data = load('-ascii',filename);

    % assign cause and effect
    cause = data(:,metadata(pair,2):metadata(pair,3));
    effect = data(:,metadata(pair,4):metadata(pair,5));

    % normalize cause and effect
    if preprocessing.normalize
      cause = normalize(cause);
      effect = normalize(effect);
    end
    
    % undiscretize
    if preprocessing.undisc
      cause = undiscretize(cause);
      effect = undiscretize(effect);
    end

    % add disturbance
    cause = cause + preprocessing.disturbance * randn(size(cause));
    effect = effect + preprocessing.disturbance * randn(size(effect));

    % apply deterministic random permutation
    N = size(data,1);
    index = myrandperm(N,preprocessing.randseed);
    % take only first N data points, if necessary
    if N > preprocessing.maxN
      index = index(1:preprocessing.maxN);
    end
    cause = cause(index,:);
    effect = effect(index,:);
    weight = metadata(pair,6);
  end

return
