function plot_results(methods,prefix,uniquenessthreshold,maxpairs,type,labels)
% function plot_results(methods,prefix,uniquenessthreshold,maxpairs,type,labels)
%
% Copyright (c) 2008-2014  Joris Mooij  <j.m.mooij@uva.nl>
% All rights reserved.  See the file LICENSE for license terms.

  if nargin < 6 || isempty(labels)
    labels = methods;
  end
  [decisions,nrpairs,weights] = read_decisions(methods,prefix,uniquenessthreshold,maxpairs,Inf);
  nrmethods = length(methods);
  totalweight = sum(weights);

  if strcmpi(type,'eachpair')
    % average decisions over reps
    for method=1:nrmethods
      for nrdec=1:nrpairs
        for pair=1:nrpairs
          decisions{method,nrdec,pair} = mean(decisions{method,nrdec,pair});
        end
      end
    end

    % average decisions over decision rate
    dec = zeros(nrmethods,nrpairs);
    for method=1:nrmethods
      for pair=1:nrpairs
        dec(method,pair) = mean([decisions{method,:,pair}]);
      end
    end

    % plot decisions
    figure;
    ax1 = subplot('Position',[0.2 0.1 0.65 0.8]);
    imagesc(dec);
    set(ax1,'YTick',[1:nrmethods]);
    set(ax1,'YTickLabel',labels);
    colormap([[linspace(1,0,100) linspace(0,0,100) linspace(0,1,100)];[linspace(0,0,100) linspace(0,1,100) linspace(0,1,100)];[linspace(0,0,100) linspace(0,0,100) linspace(0,1,100)]]');
    caxis(ax1,[-1,2]);
    xlabel(ax1,'Cause-Effect Pair #');
    title('Green: correct;  Red: wrong;  Yellow: undecided');

    % plot weights
    ax2 = subplot('Position',[0.2 0.925 0.65 0.05]);
    imagesc(weights'+1);
    caxis(ax2,[-1,2]);
    set(ax2,'XTick',[]);
    set(ax2,'XTickLabel','');
    set(ax2,'YTick',[1]);
    set(ax2,'YTickLabel','Weights');
    xlabel('');
    ylabel('');

    % plot summaries
    sorteddec = sort(dec')';
    ax3 = subplot('Position',[0.9 0.1 0.1 0.8]);
    imagesc(sorteddec);
    caxis(ax3,[-1,2]);
    set(ax3,'XTick',[]);
    set(ax3,'XTickLabel','');
    set(ax3,'YTick',[]);
    set(ax3,'YTickLabel','');
    xlabel('');
    ylabel('');

    % calculate accuracies in forced-decision scenario
    for method = 1:nrmethods
      total = sum((~isnan(dec(method,:))) .* weights');
      right = sum((dec(method,:) > 0) .* weights');
      wrong = sum((dec(method,:) < 0) .* weights');
      fprintf('%s: total %3.1f, right %3.1f, wrong %3.1f, undecided %3.1f, accuracy %3.1f\n',labels{method},total,right,wrong,total-right-wrong,right/(right+wrong)*100);
      t=text(size(sorteddec,2)-1,method,sprintf('%3.1f%%',right/(right+wrong)*100));
      set(t,'HorizontalAlignment','Right');
      set(t,'Color',[1 1 1]);
    end
  elseif strcmpi(type,'curves') || strcmpi(type,'shapes')
    % merge pairs
    alpha = 1 - (normcdf(1,0,1) - normcdf(-1,0,1));  % confidence interval corresponding to about 2 standard deviations
    total = cell(nrmethods,nrpairs);
    right = cell(nrmethods,nrpairs);
    wrong = cell(nrmethods,nrpairs);
    for method = 1:nrmethods
      for nrdec = 1:nrpairs
        reps = length(decisions{method,nrdec,1});
        decs = [decisions{method,nrdec,:}];
        for rep = 1:reps
          dec = decs(rep,:);
          right{method,nrdec} = [right{method,nrdec}; (dec ==  1) * weights];
          wrong{method,nrdec} = [wrong{method,nrdec}; (dec == -1) * weights];
        end
        % calculate statistics
        total{method,nrdec} = right{method,nrdec} + wrong{method,nrdec};
        [phats, pcis] = binofit(ceil(right{method,nrdec}),ceil(total{method,nrdec}));
        phat{method,nrdec} = phats;
        pcil{method,nrdec} = pcis(:,1);
        pciu{method,nrdec} = pcis(:,2);
        [dummy, pcisstd] = binofit(ceil(right{method,nrdec}),ceil(total{method,nrdec}),alpha);
        perr{method,nrdec} = (pcisstd(:,2) - pcisstd(:,1)) / 2;
        pcistdl{method,nrdec} = pcisstd(:,1);
        pcistdu{method,nrdec} = pcisstd(:,2);
      end
    end

    % plot curves
    figure;
    xlim([0 100]);
    ylim([0 101]);
    set(gca,'FontSize',12);
    hold on
    co = get(gca,'ColorOrder');

    if strcmpi(type,'curves')
      itw = round(totalweight);
      X = 100 * [1:itw] / itw;
      Y = 100 * binoinv(0.975*ones(1,itw),[1:itw],0.5*ones(1,itw)) ./ [1:itw];
      bla = fill([X';flipud(X')],[Y';flipud(100-Y')],[0.5 0.5 0.5]);
      set(bla,'EdgeColor',[0.3 0.3 0.3]);
%      set(bla,'FaceAlpha',0.5);
%      set(bla,'EdgeAlpha',0.7);
      h = text(95,90,'Significant');
      set(h,'HorizontalAlignment','right');
      h = text(95,55,'Not significant');
      set(h,'HorizontalAlignment','right');
    end
    plot([0 100],[50 50],'k:');

    p = [];
    for method=1:nrmethods
      reps = length(decisions{method,1,1});
      % arrange different reps into matrix
      totals = [total{method,:}];
      rights = [right{method,:}];
      phats = [phat{method,:}];
      pcils = [pcil{method,:}];
      pcius = [pciu{method,:}];
      pcistdls = [pcistdl{method,:}];
      pcistdus = [pcistdu{method,:}];
      decrates = totals / totalweight;
      assert(sum(sum(isnan(decrates))) == 0);
      assert(sum(sum(isnan(phats))) == 0);
      assert(sum(sum(isnan(pcils))) == 0);
      assert(sum(sum(isnan(pcius))) == 0);

      fprintf('Forced-choice accuracy: %f+-%f (95%% confidence interval [%f,%f], 68%% confidence interval [%f,%f])\n',mean(phat{method,nrpairs}),mean(perr{method,nrpairs}),mean(pcil{method,nrpairs}),mean(pciu{method,nrpairs}),mean(pcistdl{method,nrpairs}),mean(pcistdu{method,nrpairs}));
      curcol = co(1+mod(method-1,size(co,1)),:);
      white = [1 1 1];
      if strcmpi(type,'shapes')
        % 95% confidence interval
        bla = fill([100*mean(decrates,1)';flipud(100*mean(decrates,1)')],[100*mean(pcils,1)';flipud(100*mean(pcius,1)')],curcol * 0.3 + 0.7 * white);
        set(bla,'EdgeColor',curcol * 0.5 + 0.5 * white);
%        set(bla,'FaceAlpha',0.3);
%        set(bla,'EdgeAlpha',0.5);
        
        % 68% confidence interval
        bla = fill([100*mean(decrates,1)';flipud(100*mean(decrates,1)')],[100*mean(pcistdls,1)';flipud(100*mean(pcistdus,1)')],curcol * 0.5 + 0.5 * white);
        set(bla,'EdgeColor',curcol * 0.7 + 0.3 * [1 1 1]);
%        set(bla,'FaceAlpha',0.5);
%        set(bla,'EdgeAlpha',0.7);
      end
      p(method) = plot(100*mean(decrates,1),100*mean(phats,1),'-','Color',curcol,'LineWidth',2);
      if 0
        for rep=1:reps
          plot(100*decrates(rep,:),100*phats(rep,:),'.','Color',curcol);
        end
      end
    end

    legend(p,labels);
    set(legend,'Location','SouthEast');
    xlabel('Decision rate (%)','FontSize',14);
    ylabel('Accuracy (%)','FontSize',14);
    hold off
  else
    error('Unknown type');
  end

return
