function [decision,conf] = run_method (pair,method,methodpars,outdir,preprocessing)
% function [decision,conf] = run_method (pair,method,methodpars,outdir,preprocessing)
%
% Wrapper that runs a causal inference method on a cause effect pair 
% and writes results to a file in outdir. It also returns its decision 
% about the most likely causal direction and its confidence value(s). 
% Since usually there is no unique confidence estimate, a method may 
% return several values.
%
% Loads variables cause,effect from the cause-effect pair, applies preprocessing and invokes:
%   [decision,conf] = method(cause,effect,methodpars,outfile)
%
% INPUT:   pair           number of the cause effect pair
%          method         function handle of the method to call (see also cep_template)
%          methodpars     parameters for the method
%          outdir         output directory
%          preprocessing  the preprocessing to apply to the data (see also load_pair)
%
% OUTPUT:  decision         decision about causal direction
%                             1    means correct
%                             -1   means wrong
%                             0    means none
%                             NAN  means error
%          conf             array of confidence values that the decision is correct.
%                           The number and precise meaning of those values depends
%                           on the method. In general, the higher the confidence,
%                           the more certain a method is. It should be -Inf in case
%                           decision == NAN.
%
% Copyright (c) 2008-2014  Joris Mooij  <j.m.mooij@uva.nl>
% All rights reserved.  See the file LICENSE for license terms.

  % print diagnostics
  fprintf('Running method %s on pair %d\n',method,pair);
  methodpars,
  preprocessing,

  % abort if destination file exists already
  outfile = sprintf('%s/pair%04d.hdf5',outdir,pair);
  if exist(outfile,'file')
    % skip if file already exists
    fprintf('%s already exists! Loading existing results...\n', outfile);
    result = load('-mat',outfile);
    decision = result.decision;
    conf = result.conf;
    return;
  end;

  % load data and apply preprocessing
  global WEBDAVDIR;
  [cause,effect] = load_pair(pair,preprocessing,WEBDAVDIR);

  % create outdir
  [success,msg] = mkdir(outdir);

  % run causal inference method
  [decision,conf] = feval(method,cause,effect,methodpars,outfile);

return
